from PySteppables import *
import CompuCell
import sys
from math import *
from random import *
from CompuCell import getPyAttrib
from PlayerPython import *
from copy import deepcopy
import time
import os
import CompuCellSetup
      
#
class DiffusionCalc(SteppableBasePy):
  def __init__(self,_simulator,_frequency,_time): 
    SteppableBasePy.__init__(self,_simulator,_frequency)
    self.time=_time
    
  def start(self):
    #---Creating file:
    import os
    SourceDir = os.path.dirname(os.path.abspath(__file__))
    self.FileName = SourceDir + "\diffusion.dat"
    #---Initializing calculations:
    for cell in self.cellListByType(self.CELLS):
      cellDict=CompuCell.getPyAttrib(cell)
      #list of cell positions (initially the list has only the original position)
      cellDict["x"]=[cell.xCOM]; cellDict["y"]=[cell.yCOM]
      #record the last position
      cellDict["xLast"]=cell.xCOM; cellDict["yLast"]=cell.yCOM
      #record of how many times the cells crossed the lattice boundary
      cellDict['xx']=0; cellDict['yy']=0
    #---Graph:
    self.pW=self.addNewPlotWindow(_title='Diffusion',_xAxisTitle='Time Window (MCS)',_yAxisTitle='<d^2> (pixels^2)')
    self.pW.addPlot('Data',_style='Dots',_color='red',_size=5)
    self.pW.addPlot('Fit',_style='Lines',_color='blue',_size=2)
    self.pW.setXAxisLogScale()
    self.pW.setYAxisLogScale()
     
  def step(self,mcs):
    #---Storing cells' positions:
    for cell in self.cellListByType(self.CELLS):
      #cell's absolute postion
      x=cell.xCOM; y=cell.yCOM
      cellDict=CompuCell.getPyAttrib(cell)
      #checking if cell crossed the border
      dx=x-cellDict['xLast']; dy=y-cellDict['yLast']
      if (dx>self.dim.x/2): cellDict['xx']-=1
      if (dx<-self.dim.x/2): cellDict['xx']+=1
      if (dy>self.dim.y/2): cellDict['yy']-=1
      if (dy<-self.dim.y/2): cellDict['yy']+=1
      #storing cell's relative position
      cellDict['x'].append(x+self.dim.x*cellDict['xx'])
      cellDict['y'].append(y+self.dim.y*cellDict['yy'])
      #storing current absolute position for next iteration
      cellDict["xLast"]=cell.xCOM; cellDict["yLast"]=cell.yCOM
    
  def finish(self):
    File=open(self.FileName,'w')
    #---calculating <x^2> vs <t>:
    dt=0.5; L={}
    while (dt<self.time/2):
      dt=dt*2; n=0; dL2=0
      print "**************** - ", dt
      for cell in self.cellListByType(self.CELLS):
        cellDict=CompuCell.getPyAttrib(cell)
        for t in range(self.time-int(dt)):
           x0=cellDict["x"][t]
           y0=cellDict["y"][t]
           xf=cellDict["x"][t+int(dt)]
           yf=cellDict["y"][t+int(dt)]
           dx=xf-x0; dy=yf-y0
           dL2+=(dx*dx+dy*dy); n+=1
      dL2=dL2/n
      #storing information on a file
      File.write("%s" % (str(dt)+", "))
      File.write("%f\n" % (dL2))
      #plotting
      self.pW.addDataPoint("Data",dt,dL2) # arguments are (name of the data series, x, y)
      self.pW.showAllPlots()
      #storing all data points
      L[log(dt)]=log(dL2)
    File.close()
    #---Calculating diffusion:
    a,b = self.linearFit(L)
    D = e**b
    alpha = a
    print "--------------------"
    print "Diffusion constant:", D, "pixels^2/mcs"
    print "Diffusion exponent:", alpha
    print "--------------------"
    #---plottig the fit:
    self.pW.addDataPoint("Fit",1,D)
    self.pW.addDataPoint("Fit",dt,D*(dt)**alpha)
    self.pW.setXAxisScale(1,self.time)
    self.pW.setYAxisScale(D/2,D*(dt*2)**alpha)
    self.pW.showAllPlots()
    
  def linearFit(self,L):
    from numpy import matrix
    xT=sum(L.keys())
    yT=sum(L.values())
    x2T=0; xy=0
    for x,y in L.items():
      x2T+=x*x; xy+=x*y
    A = matrix([ [len(L),xT],[xT,x2T] ])
    B = matrix([ [yT],[xy] ])
    b,a = A.I*B
    a=float(a); b=float(b)
    return a,b